<?php

namespace Drupal\responsive_image_class\Plugin\Field\FieldFormatter;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Utility\LinkGeneratorInterface;
use Drupal\responsive_image\Plugin\Field\FieldFormatter\ResponsiveImageFormatter;
use Drupal\responsive_image_class\ClassFormatter;
use Drupal\responsive_image_class\FocalPoint;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin for responsive image formatter.
 *
 * @FieldFormatter(
 *   id = "responsive_image_class",
 *   label = @Translation("Responsive image class"),
 *   field_types = {
 *     "image",
 *   },
 *   quickedit = {
 *     "editor" = "image"
 *   }
 * )
 */
class ResponsiveImageClassFormatter extends ResponsiveImageFormatter {

  /**
   * The Class Formatter.
   *
   * @var \Drupal\responsive_image_class\ClassFormatter
   */
  protected $classFormatter;

  /**
   * The Focal Point service.
   *
   * @var \Drupal\responsive_image_class\FocalPoint
   */
  protected $focalPoint;

  /**
   * Constructs a ResponsiveImageFormatter object.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\Core\Entity\EntityStorageInterface $responsive_image_style_storage
   *   The responsive image style storage.
   * @param \Drupal\Core\Entity\EntityStorageInterface $image_style_storage
   *   The image style storage.
   * @param \Drupal\Core\Utility\LinkGeneratorInterface $link_generator
   *   The link generator service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\responsive_image_class\ClassFormatter $class_formatter
   *   The class formatter service for this module.
   * @param \Drupal\responsive_image_class\FocalPoint $focal_point
   *   The class for identifying the front-end focal point.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, EntityStorageInterface $responsive_image_style_storage, EntityStorageInterface $image_style_storage, LinkGeneratorInterface $link_generator, AccountInterface $current_user, ClassFormatter $class_formatter, FocalPoint $focal_point) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings, $responsive_image_style_storage, $image_style_storage, $link_generator, $current_user);
    $this->classFormatter = $class_formatter;
    $this->focalPoint = $focal_point;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('entity_type.manager')->getStorage('responsive_image_style'),
      $container->get('entity_type.manager')->getStorage('image_style'),
      $container->get('link_generator'),
      $container->get('current_user'),
      $container->get('responsive_image_class.class_formatter'),
      $container->get('responsive_image_class.focal_point')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'image_classes' => '',
      'remove_alt' => FALSE,
      'focal_point' => FALSE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);

    $elements['image_classes'] = [
      '#title' => t('Image classes'),
      '#type' => 'textfield',
      '#default_value' => $this->getSetting('image_classes') ?: '',
      '#description' => 'One or more space seperated classes to add to the image',
    ];

    $elements['focal_point'] = [
      '#title' => t('Include focus point data'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('focal_point') ? TRUE : FALSE,
      '#description' => 'Add a data-focal-position data attribute',
    ];

    $elements['remove_alt'] = [
      '#title' => t('Remove alt tag'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('remove_alt') ? TRUE : FALSE,
      '#description' => 'Use an empty alt attribute when displaying the image in this view mode',
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();

    if ($this->getSetting('image_classes')) {
      $summary[] = t('Image classes: @image_classes', ['@image_classes' => $this->getSetting('image_classes')]);
    }
    if ($this->getSetting('remove_alt')) {
      $summary[] = t('Decorative image - remove alt attribute');
    }
    if ($this->getSetting('focal_point')) {
      $summary[] = t('Show focal point data');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = parent::viewElements($items, $langcode);
    if ($this->getSetting('image_classes')) {
      foreach ($elements as &$element) {
        if ($this->getSetting('remove_alt')) {
          $element['#item']->set('alt', '');
        }
        if ($this->getSetting('image_classes')) {
          $element['#item_attributes']['class'] = $this->classFormatter->classStringToArray($element, $this->getSetting('image_classes'));
        }
        if ($this->getSetting('focal_point')) {
          $element['#item_attributes']['data-focal-position'] = FocalPoint::DEFAULT_POSITION;
          $value = $element['#item']->getValue();
          if (isset($value['width']) && isset($value['height']) && isset($element['#item']->entity)) {
            $element['#item_attributes']['data-focal-position'] = $this->focalPoint->getFocalPosition($element['#item']->entity, $value['width'], $value['height']);
          }
        }
      }
    }
    return $elements;
  }

}
